/*
 *	wmnut-0.09 : rcfiles.c
 *
 *              These are usefull functions to manage with rc files  
*
 *
 * 	This program is free software; you can redistribute it and/or modify
 * 	it under the terms of the GNU General Public License as published by
 * 	the Free Software Foundation; either version 2, or (at your option)
 * 	any later version.
 *
 * 	This program is distributed in the hope that it will be useful,
 * 	but WITHOUT ANY WARRANTY; without even the implied warranty of
 * 	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * 	GNU General Public License for more details.
 *
 * 	You should have received a copy of the GNU General Public License
 * 	along with this program (see the file COPYING); if not, write to the
 * 	Free Software Foundation, Inc.,
 * 	59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 */


/*******************************************************************************\
|* AddRcKey		  						        |
\*******************************************************************************/
void AddRcKey(rckeys *key, const char *label, int type, void *var) {

  if(label != NULL) {
    key->label = (char *)malloc(strlen(label) + 1);
    strcpy(key->label, label);
  }
  else
    key->label = NULL;

  key->type = type;

  switch(type) {
  case TYPE_STRING :
    key->var.str = (char *)var;
    break;
  case TYPE_BOOL: 
  case TYPE_INT:
    key->var.integer = (int *)var;
    break;
  case TYPE_FLOAT:
    key->var.floater = (float *)var;
    break;
  case TYPE_NULL:
    break;
  }
}

/*******************************************************************************\
|* ParseRCFile		  						        |
\*******************************************************************************/
void ParseRCFile(const char *filename, rckeys *keys) {

  char	*p, *tmp;
  char	temp[128];
  char	*tokens = " =\t\n#";
  FILE	*fp;
  int	key;
  
  fp = fopen(filename, "r");
  if (fp) {
#ifdef DEBUG
    printf("Opening rc file %s\n", filename);
#endif
    while (fgets(temp, 128, fp)) {
      if (temp[0] != '#') {
	key = 0;
	while (key >= 0 && keys[key].label) {
	  if ((p = strstr(temp, keys[key].label))) {
	    /* jump to "=" */
	    p += strlen(keys[key].label);
	    /* suppress tokens before value */
	    p += strspn(p, tokens);
	    /* suppress tokens after value */
	    tmp = strpbrk(p, tokens);
	     *tmp = '\0';
#ifdef DEBUG
	    printf("Parameter -> %s%s\tValue -> %s\n", keys[key].label, 
		   ((strlen(keys[key].label) <12)?"\t\t":"\t"), p);
#endif
	    switch(keys[key].type) {
	    case TYPE_STRING :
	      strncpy(keys[key].var.str, p, strlen(p));
	      break;
	    case TYPE_BOOL:    
	      if (!strncmp(p, "on", 2))
		*keys[key].var.bool = 1;
	      else
		*keys[key].var.bool = 0;
	      break;
	    case TYPE_INT:
	      *keys[key].var.integer = atoi(p);
	      break;
	    case TYPE_FLOAT:
	      *keys[key].var.floater = atof(p);
	      break;
	    }
	    key = -1;
	  } else key++;
	}
      }
    }
    fclose(fp);
  }
}

/*******************************************************************************\
|* LoadRCFile		  						        |
\*******************************************************************************/
void LoadRCFile(rckeys *keys)
{
  char	home_file[128];
  char	*p;

#ifdef DEBUG
    printf("Loading rc files\n");
#endif

  ParseRCFile(MAINRC_FILE, keys);

  p = getenv("HOME");
  sprintf(home_file, "%s/%s", p, RC_FILE);
  ParseRCFile(home_file, keys);

}

/*******************************************************************************\
|* ReloadRCFile		  						        |
\*******************************************************************************/
void ReloadRCFile()
{

#ifdef DEBUG
    printf("Reloading rc files\n");
#endif
/*      LoadRCFile(); */
}
